--Set max flashtics, 80 default should be about mid between vanilla and KMP
--Can't go below vanilla (~53)
--Make sure kmp_extendflashtics is enabled for this to work
local kmph_flashcap = CV_RegisterVar({
	name = "kmph_flashcap",
	defaultvalue = 80,
	flags = CV_NETVAR,
	PossibleValue = {MIN = 53, MAX = 999999999},
})

--Toggles adding player's kartspeed as an adjustment to flashtic length
--Vanilla default is on
local kmph_flashspeedadjust = CV_RegisterVar({
	name = "kmph_flashspeedadjust",
	defaultvalue = "On",
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff,
})

--Toggle allowing KMPH to handle hyudoro
--MAKE SURE KMP_HYUDORO IS OFF
local kmph_hyudoro = CV_RegisterVar({
	name = "kmph_hyudoro",
	defaultvalue = "Off",
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff,
})

--Adjustable tic definition for the length of Hyudoro invuln
-- 244 - Default vanilla length
local kmph_hyulength = CV_RegisterVar({
	name = "kmph_hyulength",
	defaultvalue = 244,
	flags = CV_NETVAR,
	PossibleValue = CV_Natural,
})

local function KH_handleflashing(p)
	--kmp's flashtic changes are not toggleable, override it here.
	--Use the same safety checks
	if CV_FindVar("kmp_extendflashtics") and not CV_FindVar("kmp_extendflashtics").value return end
	if gametype ~= GT_RACE return end
	if leveltime < TICRATE*8 return end
	if not p.mo return end
	
	--Reset overwrite flag
	if p.mo.K_cansetflashtics and p.mo.KH_flashticoverwritten then
		--print("KMP Flash - Overwrite flag reset")
		p.mo.KH_flashticoverwritten = nil
	end
	
	--Detect when KMP sets flashing and overwrite it's value
	if p.powers[pw_flashing] and not p.kartstuff[k_hyudorotimer] and not p.mo.K_cansetflashtics and not p.mo.KH_flashticoverwritten
		if kmph_flashspeedadjust.value then
			p.mo.K_fakeflashing = kmph_flashcap.value + (kmph_flashcap.value / 8) * (p.kartspeed)
		else
			p.mo.K_fakeflashing = kmph_flashcap.value
		end		
		p.mo.KH_flashticoverwritten = true	
		--print("KMP Flash - Overwritten with value of "..tostring(p.mo.K_fakeflashing))
	end
end

addHook("MobjThinker", function(mo)
	if kmph_hyudoro.value and mo and mo.valid and mo.target and mo.target.valid and mo.target.player
	and mo.target.player.kartstuff[k_hyudorotimer]	-- !?
		mo.target = nil
	end
end, MT_JAWZ)

local serverwarn = false
local function KH_handleHyudoro(p)
	if not kmph_hyudoro.value return end
	if CV_FindVar("kmp_hyudoro") and CV_FindVar("kmp_hyudoro").value == 1 and not serverwarn then
		print("WARNING: Do not enable kmph_hyudoro and kmp_hyudoro at the same time. Only use one.")
		serverwarn = true
		return
	end
	
	if p.kartstuff[k_hyudorotimer] then
		if not p.kh_hyudoroextend then
			p.kartstuff[k_hyudorotimer] = kmph_hyulength.value
			p.kh_hyudoroextend = true
		end
	else
		p.kh_hyudoroextend = nil
	end
end

addHook("ThinkFrame", do
	for p in players.iterate do
		KH_handleflashing(p)
		KH_handleHyudoro(p)
	end
end)